import json
from typing import List

from rubrical.enum import DependencySpecifications, SupportedPackageManagers
from rubrical.package_managers.base_package_manager import BasePackageManager
from rubrical.schemas.package import Package, Specification


class NodeJS(BasePackageManager):
    target_files = ["package.json"]

    def __init__(self) -> None:
        super().__init__()

        self.name = SupportedPackageManagers.NODEJS.value

        self.specification_symbols = self.specification_symbols | {
            DependencySpecifications.APPROX_EQ.value: ["~"],
            DependencySpecifications.COMPATIBLE.value: ["^"],
        }

    def _parse_packagelock(
        self, package_file_filename: str, package_file_contents: str
    ):
        self.packages[package_file_filename] = []
        package_json = json.loads(package_file_contents)

        for dependency_key in [
            x for x in package_json.keys() if "dependencies" in x.lower()
        ]:
            dependency_section = package_json[dependency_key]

            for package, version in dependency_section.items():
                if " " not in version:  # Handle most cases through symbol matching
                    if version[-2:] == ".x":  # Handle 1.2.x
                        self.packages[package_file_filename].append(
                            Package(
                                name=package,
                                raw_constraint=f"{package} {version}",
                                version_constraints=[
                                    Specification(
                                        version=version[:-2],
                                        specifier=DependencySpecifications.APPROX_EQ,
                                    )
                                ],
                            )
                        )
                    else:
                        (
                            specifier,
                            sanitized_version,
                        ) = self.match_from_specification_symbols(version)
                        self.packages[package_file_filename].append(
                            Package(
                                name=package,
                                raw_constraint=f"{package} {version}",
                                version_constraints=[
                                    Specification(
                                        version=sanitized_version, specifier=specifier
                                    )
                                ],
                            )
                        )
                elif (
                    "||" in version
                ):  # Not sure if we can programatically parse this. :'(
                    pass
                elif "-" in version:  # Handle range
                    range_versions = [x.strip(" ") for x in version.split("-")]
                    self.packages[package_file_filename].append(
                        Package(
                            name=package,
                            raw_constraint=f"{package} {version}",
                            version_constraints=[
                                Specification(
                                    version=range_versions[0],
                                    specifier=DependencySpecifications.GTE,
                                ),
                                Specification(
                                    version=range_versions[1],
                                    specifier=DependencySpecifications.LTE,
                                ),
                            ],
                        )
                    )
                else:  # Handle standard range based definitions
                    range_versions = version.split(" ")
                    if len(range_versions) != 2:
                        pass  # Tomfoolery
                    else:
                        version_constraints: List[Specification] = []

                        for parsed_version in range_versions:
                            (
                                specifier,
                                sanitized_version,
                            ) = self.match_from_specification_symbols(parsed_version)

                            version_constraints.append(
                                Specification(
                                    version=sanitized_version,
                                    specifier=specifier,
                                )
                            )

                        self.packages[package_file_filename].append(
                            Package(
                                name=package,
                                raw_constraint=f"{package} {version}",
                                version_constraints=version_constraints,
                            )
                        )

    def parse_package_manager_file(
        self, package_file_filename: str, package_file_contents: str
    ) -> None:
        if "package.json" in package_file_filename:
            self._parse_packagelock(package_file_filename, package_file_contents)



def repository_from_url(url: str) -> str:
    repository_name = ""

    if url.startswith("https"):
        repository_name = "/".join(url[0:-4].split("/")[-2:])
    elif url.startswith("git"):
        repository_name = url.split(":")[1][:-4]

    return repository_name



from dataclasses import dataclass

from rubrical.enum import PackageCheck


@dataclass
class PackageCheckResult:
    name: str
    file: str
    check: PackageCheck
    version_package: str
    version_block: str
    version_warn: str



from enum import Enum


class SemverComparison(Enum):
    LT = -1
    GT = 1
    EQ = 0


class DependencySpecifications(Enum):
    EQ = "EQ"
    GT = "GT"
    GTE = "GTE"
    LT = "LT"
    LTE = "LTE"
    NE = "NE"
    COMPATIBLE = "COMPATIBLE"
    APPROX_EQ = "APPROX_EQ"


class PackageTypes(Enum):
    SEMVER = "semver"
    GENERIC = "generic"


class PackageCheck(Enum):
    OK = "ok"
    WARN = "warn"
    BLOCK = "block"
    NOOP = "noop"


class SupportedPackageManagers(Enum):
    JSONNET = "jsonnet"
    PYTHON = "python"
    GO = "go"
    NODEJS = "nodejs"



from dataclasses import dataclass
from typing import List

from rubrical.enum import DependencySpecifications


@dataclass
class Specification:
    version: str
    specifier: DependencySpecifications


@dataclass
class Package:
    name: str
    raw_constraint: str
    version_constraints: List[Specification]



from typing import List

import pyproject_parser
import requirements
import tomllib
from requirements.requirement import Requirement

from rubrical.enum import DependencySpecifications, SupportedPackageManagers
from rubrical.package_managers.base_package_manager import BasePackageManager
from rubrical.schemas.package import Package, Specification


class Python(BasePackageManager):
    target_files = ["requirements.txt", "pyproject.toml"]

    def __init__(self) -> None:
        super().__init__()

        self.name = SupportedPackageManagers.PYTHON.value

        self.specification_symbols = self.specification_symbols | {
            DependencySpecifications.APPROX_EQ.value: ["~="]
        }

    def _parse_requirement(self, req: Requirement, package_file_filename: str):
        if req.specifier and req.specs:
            version_constraints: List[Specification] = []

            for spec in req.specs:
                specifier = DependencySpecifications.EQ
                for key, value in self.specification_symbols.items():
                    if spec[0] in value:
                        specifier = DependencySpecifications[key]

                version_constraints.append(
                    Specification(
                        version=spec[1],
                        specifier=specifier,
                    )
                )

            self.append_package(
                package_file_filename,
                Package(
                    name=req.name,
                    raw_constraint=req.line,
                    version_constraints=version_constraints,
                ),
            )

    def parse_package_manager_file(
        self, package_file_filename: str, package_file_contents: str
    ) -> None:
        self.packages[package_file_filename] = []

        if "requirements.txt" in package_file_filename:
            for req in requirements.parse(package_file_contents):
                self._parse_requirement(
                    req=req, package_file_filename=package_file_filename
                )

        elif "pyproject.toml" in package_file_filename:
            pyproject_contents = tomllib.loads(package_file_contents)
            parser = pyproject_parser.PyProject()
            contents = parser.from_dict(pyproject_contents)

            # Is not a Poetry file.
            if contents.project and "dependencies" in contents.project.keys():
                for pyproject_req in contents.project["dependencies"]:
                    # Hacky hack to use same parser as requirements.txt
                    for fake_req in requirements.parse(str(pyproject_req)):
                        self._parse_requirement(fake_req, package_file_filename)



import re

from rubrical.enum import DependencySpecifications, SupportedPackageManagers
from rubrical.package_managers.base_package_manager import BasePackageManager
from rubrical.schemas.package import Package, Specification


class Go(BasePackageManager):
    target_files = ["go.mod"]

    def __init__(self) -> None:
        super().__init__()

        self.name = SupportedPackageManagers.GO.value

    def parse_package_manager_file(
        self, package_file_filename: str, package_file_contents: str
    ) -> None:
        self.packages[package_file_filename] = []
        modules = []

        go_import_regex = re.compile("\\(([^)]+)\\)")
        matched_groups = go_import_regex.findall(package_file_contents)
        for group in matched_groups:
            lines = group.split("\n")
            modules.extend(
                [x.replace("\t", "") for x in lines if x and "indirect" not in x]
            )

        for module in modules:
            module_specs = module.split(" ")

            self.packages[package_file_filename].append(
                Package(
                    name=module_specs[0],
                    raw_constraint=module,
                    version_constraints=[
                        Specification(
                            version=module_specs[1],
                            specifier=DependencySpecifications.EQ,
                        )
                    ],
                )
            )



from typing import List, Optional

from pydantic import BaseModel, ConfigDict

from rubrical.enum import PackageTypes, SupportedPackageManagers


class PackageRequirement(BaseModel):
    name: str
    type: Optional[PackageTypes] = PackageTypes.SEMVER
    warn: str
    block: str


class PackageManager(BaseModel):
    model_config = ConfigDict(use_enum_values=True)
    name: SupportedPackageManagers
    packages: List[PackageRequirement]


class RubricalConfig(BaseModel):
    version: int
    blocking_mode: Optional[bool] = True
    package_managers: List[PackageManager]



import json
from typing import List, Optional

from pydantic import BaseModel

from rubrical.enum import DependencySpecifications, SupportedPackageManagers
from rubrical.package_managers.base_package_manager import BasePackageManager
from rubrical.package_managers.utilities import git
from rubrical.schemas.package import Package, Specification


class JsonnetDependencySourceGit(BaseModel):
    remote: str
    subdir: Optional[str] = ""


class JsonnetDependencySource(BaseModel):
    git: JsonnetDependencySourceGit


class JsonnetDependency(BaseModel):
    source: JsonnetDependencySource
    version: str


class JsonnetFile(BaseModel):
    dependencies: List[JsonnetDependency]


class Jsonnet(BasePackageManager):
    target_files = ["jsonnetfile.json"]

    def __init__(self) -> None:
        super().__init__()

        self.name = SupportedPackageManagers.JSONNET.value

    def parse_package_manager_file(
        self, package_file_filename: str, package_file_contents: str
    ) -> None:
        file_contents = json.loads(package_file_contents)
        self.packages[package_file_filename] = []

        jsonnet_file = JsonnetFile(**file_contents)

        for dependency in jsonnet_file.dependencies:
            self.packages[package_file_filename].append(
                Package(
                    name=git.repository_from_url(dependency.source.git.remote),
                    raw_constraint=f"{git.repository_from_url(dependency.source.git.remote)} {dependency.version}",
                    version_constraints=[
                        Specification(
                            version=dependency.version,
                            specifier=DependencySpecifications.EQ,
                        )
                    ],
                )
            )



from rubrical.enum import PackageCheck, PackageTypes
from rubrical.schemas.configuration import PackageRequirement
from rubrical.schemas.package import Package

from . import semversion, string


def check_package(
    package_requirement: PackageRequirement, package: Package
) -> PackageCheck:
    result = None

    if package_requirement.type == PackageTypes.SEMVER:
        result = semversion.compare_package_semver(
            package=package, package_requirement=package_requirement
        )
    elif package_requirement.type == PackageTypes.GENERIC:
        result = string.compare_package_str(
            package=package, package_requirement=package_requirement
        )
    else:
        raise NotImplementedError("Comparison type not implemented.")

    return result



import rich
import typer


def print_header(message: str, emoji: str = "") -> None:
    rich.print(f"[bold dodger_blue1]{f'{emoji} '}{message}[/bold dodger_blue1]")


def print_message(message: str, emoji: str = "") -> None:
    rich.print(f"[steel_blue1]{f'{emoji} '}{message}[/steel_blue1]")


def print_raw(message: str, emoji: str = "") -> None:
    rich.print(f"{f'{emoji} '}{message}")


def print_debug(message: str) -> None:
    rich.print(f"Debug: {message}")


def print_error(message: str, emoji: str = "") -> None:
    rich.print(f"[bold bright_red]{f'{emoji} '}{message}[/bold bright_red]")
    raise typer.Exit(code=1)



import json
from pathlib import Path

import typer
from benedict import benedict
from pydantic import ValidationError

from rubrical.schemas.configuration import RubricalConfig
from rubrical.utilities import console

app = typer.Typer()


@app.command()
def validate(
    config: Path = typer.Option(Path("rubrical.yaml"), help="Path to configuration"),
):
    """
    Validates rubrical config.
    """
    console.print_message("Loading configuration.", "📃")
    if config.suffix in [".yaml", ".json", ".toml"]:
        try:
            RubricalConfig(**benedict(config, format=(config.suffix[1:])))
        except ValidationError as e:
            console.print_raw(str(e))
            console.print_error("Configuration error found!", "🔴")
    else:
        raise ValueError(
            "Rubrical only supports YAML, JSON, or TOML configuration files"
        )

    console.print_message("Configuration is OK!", "✅")


@app.command()
def jsonschema():
    """
    Prints configuration jsonschema.
    """
    console.print_message(json.dumps(RubricalConfig.model_json_schema(), indent=2))



from typing import List

from rich.console import Console
from rich.table import Table

from rubrical.enum import PackageCheck
from rubrical.schemas.results import PackageCheckResult
from rubrical.utilities import console


def terminal_report(
    package_manager_name: str, check_results: List[PackageCheckResult]
) -> None:
    # If any unsuccessful checks.
    if [x for x in check_results if x.check in [PackageCheck.BLOCK, PackageCheck.WARN]]:
        console.print_message(
            f"[bold][dark_orange]{package_manager_name}[/dark_orange][/bold] checks completed with violations!"
        )

        rconsole = Console()

        table = Table("File", "Dependency", "Result")

        # Only report unsuccessful checks.
        for result in [x for x in check_results if x.check != PackageCheck.OK]:
            if result.check == PackageCheck.BLOCK:
                result_text = f"❌ {result.version_package} < {result.version_block}, update to >= {result.version_warn}"
            elif result.check == PackageCheck.WARN:
                # Terminal is cranky about the emoji, needs two spaces.
                result_text = f"🚧 {result.version_package} < {result.version_warn}, update to >= {result.version_warn}"

            table.add_row(result.file, result.name, result_text)

        rconsole.print(table)
    else:
        console.print_message(
            f"[bold][spring_green1]{package_manager_name}[/spring_green1][/bold] checks completed with no warnings or blocks!"
        )



from typing import Dict, List

from github import Github

from rubrical.enum import PackageCheck
from rubrical.schemas.results import PackageCheckResult
from rubrical.utilities import console


def _generate_report(reporting_data: Dict[str, List[PackageCheckResult]]):
    test = """
## [Rubrical](https://github.com/ivanklee86/rubrical) Report

"""

    for package_manager in reporting_data.keys():
        test += f"### {package_manager}\n\n"

        not_ok_results = [
            x
            for x in reporting_data[package_manager]
            if x.check in [PackageCheck.BLOCK, PackageCheck.WARN]
        ]
        if not_ok_results:
            test += "| File | Dependency | Result |\n"
            test += "|------|------------|--------|\n"

            for result in not_ok_results:
                if result.check == PackageCheck.BLOCK:
                    test += f"| {result.file} | {result.name} | ❌ {result.version_package} < {result.version_block}, update to >= {result.version_warn} |\n"
                elif result.check == PackageCheck.WARN:
                    test += f"| {result.file} | {result.name} | ⚠️ {result.version_package} < {result.version_warn}, update to >= {result.version_warn} |\n"
        else:
            test += "🟢 All dependencies up to date!"

    return test


def report_github(
    access_token: str,
    custom_url: str,
    repository_name: str,
    pr_id: int,
    reporting_data: Dict[str, List[PackageCheckResult]],
    warnings_found: bool,
    blocks_found: bool,
):
    # Set up Github
    if custom_url:
        g = Github(base_url=f"{custom_url}/api/v3", login_or_token=access_token)
    else:
        g = Github(access_token)
    repo = g.get_repo(repository_name)
    pr = repo.get_pull(pr_id)

    # Find comment if it exists.
    rubrical_comment = None
    for issue_comment in pr.get_issue_comments():
        if (
            "[Rubrical](https://github.com/ivanklee86/rubrical) Report"
            in issue_comment.body
        ):
            rubrical_comment = issue_comment

    if warnings_found or blocks_found:
        console.print_message("Posting results to Github comment.")
        if rubrical_comment:
            rubrical_comment.edit(_generate_report(reporting_data))
        else:
            pr.create_issue_comment(_generate_report(reporting_data))
    elif rubrical_comment:
        console.print_message("Cleaning up old report.")
        rubrical_comment.delete()
    else:
        console.print_message("Nothing to report, skipping Github.")



from typing import List

from rubrical.enum import PackageCheck


def results_to_status(warn_signal: bool, block_signal: bool) -> PackageCheck:
    if block_signal:
        status = PackageCheck.BLOCK
    elif warn_signal:
        status = PackageCheck.WARN
    else:
        status = PackageCheck.OK

    return status


def max_status(results: List[PackageCheck]):
    for value in [
        PackageCheck.BLOCK,
        PackageCheck.WARN,
        PackageCheck.OK,
        PackageCheck.NOOP,
    ]:
        if value in results:
            return value



from typing import List

import semver

from rubrical.comparisons.utils import max_status, results_to_status
from rubrical.enum import (
    DependencySpecifications,
    PackageCheck,
    SemverComparison,
)
from rubrical.schemas.configuration import PackageRequirement
from rubrical.schemas.package import Package, Specification


def __semver_conversion(version: str):
    return version[1:] if version.startswith("v") else version


def __evaluate_specification(
    package_requirement: PackageRequirement, specification: Specification
) -> PackageCheck:
    result = None

    if specification.specifier in [
        DependencySpecifications.EQ,
        DependencySpecifications.LT,
        DependencySpecifications.LTE,
    ]:
        package_semver = __semver_conversion(specification.version)
    elif specification.specifier in [
        DependencySpecifications.NE,
        DependencySpecifications.GT,
        DependencySpecifications.GTE,
    ]:
        result = PackageCheck.NOOP
    elif specification.specifier == DependencySpecifications.APPROX_EQ:
        parsed_version = __semver_conversion(specification.version).split(".")
        if len(parsed_version) < 3:
            parsed_version += ["999999"] * (3 - len(parsed_version))
        package_semver = ".".join(parsed_version)
    elif specification.specifier == DependencySpecifications.COMPATIBLE:
        parsed_version = __semver_conversion(specification.version).split(".")
        for idx in reversed(range(len(parsed_version))):
            if parsed_version[idx]:
                parsed_version[idx] = "999999"
                break
        if len(parsed_version) < 3:
            parsed_version += ["999999"] * (3 - len(parsed_version))
        package_semver = ".".join(parsed_version)

    if not result:
        try:
            warn_signal = SemverComparison.LT.value == semver.compare(
                package_semver,
                __semver_conversion(package_requirement.warn),
            )
            block_signal = SemverComparison.LT.value == semver.compare(
                package_semver,
                __semver_conversion(package_requirement.block),
            )
        except ValueError:
            warn_signal = False
            block_signal = False

        result = results_to_status(warn_signal=warn_signal, block_signal=block_signal)

    return result


def compare_package_semver(
    package_requirement: PackageRequirement, package: Package
) -> PackageCheck:
    raw_result: List[PackageCheck] = list(
        map(
            lambda x: __evaluate_specification(package_requirement, x),
            package.version_constraints,
        )
    )

    return max_status(raw_result)



import abc
from pathlib import Path
from typing import Dict, List, Tuple

from rubrical.enum import DependencySpecifications
from rubrical.schemas.package import Package


class BasePackageManager(abc.ABC):
    name: str = ""
    target_files: List[str] = []
    found_files: Dict[str, str]
    packages: Dict[str, List[Package]]
    specification_symbols: Dict[str, List[str]] = {
        DependencySpecifications.EQ.value: ["=="],
        DependencySpecifications.GT.value: [">"],
        DependencySpecifications.GTE.value: [">=", "=>"],
        DependencySpecifications.LT.value: ["<"],
        DependencySpecifications.LTE.value: ["<=", "=<"],
        DependencySpecifications.NE.value: ["!="],
        DependencySpecifications.COMPATIBLE.value: [],
        DependencySpecifications.APPROX_EQ.value: [],
    }

    def __init__(self) -> None:
        super().__init__()

        self.found_files = {}
        self.packages = {}

    def read_package_manager_files(self, current_folder: Path):
        package_manager_files: List[Path] = []

        for target_file in self.target_files:
            package_manager_files.extend(current_folder.rglob(target_file))

        for package_manager_file in package_manager_files:
            with open(str(package_manager_file), "r") as file:
                self.found_files[
                    str(package_manager_file.relative_to(current_folder))
                ] = file.read()

    def parse_package_manager_files(self):
        for file in self.found_files.items():
            self.parse_package_manager_file(*file)

    def append_package(self, package_file_filename: str, package: Package):
        self.packages[package_file_filename].append(package)

    def match_from_specification_symbols(
        self, version: str
    ) -> Tuple[DependencySpecifications, str]:
        specficiation = DependencySpecifications.EQ
        sanitized_version = version

        for key, specification_symbols in self.specification_symbols.items():
            if any(symbol in version for symbol in specification_symbols):
                specficiation = DependencySpecifications[key]

                for symbol in specification_symbols:
                    for symbol_char in symbol:
                        sanitized_version = sanitized_version.replace(symbol_char, "")

        return (specficiation, sanitized_version)

    @abc.abstractmethod
    def parse_package_manager_file(
        self, package_file_filename: str, package_file_contents: str
    ) -> None:
        pass



from rubrical.comparisons.utils import results_to_status
from rubrical.enum import DependencySpecifications, PackageCheck
from rubrical.schemas.configuration import PackageRequirement
from rubrical.schemas.package import Package


def compare_package_str(
    package_requirement: PackageRequirement, package: Package
) -> PackageCheck:
    if (
        len(package.version_constraints) > 1
        or package.version_constraints[0].specifier != DependencySpecifications.EQ
    ):
        return PackageCheck.NOOP

    warn_signal = package_requirement.warn > package.version_constraints[0].version
    block_signal = package_requirement.block > package.version_constraints[0].version

    return results_to_status(warn_signal=warn_signal, block_signal=block_signal)



from pathlib import Path

import typer
from benedict import benedict
from pydantic import ValidationError

from rubrical.reporters import gh
from rubrical.rubrical import Rubrical
from rubrical.schemas.configuration import RubricalConfig
from rubrical.subcommands import configs
from rubrical.utilities import console

app = typer.Typer()
app.add_typer(configs.app, name="configs")


@app.command()
def grade(
    config: Path = typer.Option(Path("rubrical.yaml"), help="Path to configuration"),
    target: Path = typer.Option(Path().absolute(), help="Path to configuration"),
    block: bool = typer.Option(True, "/--no-block", help="Don't fail if blocks found."),
    repository_name: str = typer.Option(
        "", envvar="RUBRICAL_REPOSITORY", help="Repository name for reporting purposes."
    ),
    pr_id: int = typer.Option(
        0, envvar="RUBRICAL_PR_ID", help="PR ID for reporting purposes."
    ),
    gh_access_token: str = typer.Option(
        "",
        envvar="RUBRICAL_GH_TOKEN",
        help="Github access token for reporting.  Presence will enable Github reporting.",
    ),
    gh_custom_url: str = typer.Option(
        "",
        envvar="RUBRICAL_GH_CUSTOM_URL",
        help="Github Enterprise custom url. e.g. https://github.custom.dev",
    ),
    debug: bool = typer.Option(
        False, envvar="RUBGRICAL_DEBUG", help="Enable debug messages"
    ),
):
    """
    A CLI to encourage (😅) people to update their dependencies!
    """

    console.print_header("Rubrical starting!", "⚙️ ")

    console.print_message("Loading configuration.", "📃")
    if config.suffix in [".yaml", ".json", ".toml"]:
        try:
            configuration = RubricalConfig(
                **benedict(config, format=(config.suffix[1:]))
            )
        except ValidationError as e:
            console.print_raw(str(e))
    else:
        raise ValueError(
            "Rubrical only supports YAML, JSON, or TOML configuration files"
        )

    rubrical = Rubrical(
        configuration=configuration, repository_path=target, debug=debug
    )
    (warnings_found, blocks_found, check_results) = rubrical.check_package_managers()

    if gh_access_token:
        gh.report_github(
            access_token=gh_access_token,
            custom_url=gh_custom_url,
            repository_name=repository_name,
            pr_id=pr_id,
            reporting_data=check_results,
            warnings_found=warnings_found,
            blocks_found=blocks_found,
        )

    if blocks_found and block:
        console.print_error("Blocked dependencies found!", "🛑")
    elif warnings_found:
        console.print_header(
            "Warnings, some dependencies may need updating soon!", "☢️ "
        )
    else:
        console.print_header("All dependencies up to date!", "🟢")


if __name__ == "__main__":
    app()



from pathlib import Path
from typing import Dict, List, Tuple

from rubrical.comparisons import check_package
from rubrical.enum import PackageCheck, SupportedPackageManagers
from rubrical.package_managers.base_package_manager import BasePackageManager
from rubrical.package_managers.go import Go
from rubrical.package_managers.jsonnet import Jsonnet
from rubrical.package_managers.nodejs import NodeJS
from rubrical.package_managers.python import Python
from rubrical.reporters import terminal
from rubrical.schemas.configuration import RubricalConfig
from rubrical.schemas.results import PackageCheckResult
from rubrical.utilities import console

PACKAGE_MANAGER_MAPPING = {
    SupportedPackageManagers.JSONNET.value: Jsonnet,
    SupportedPackageManagers.PYTHON.value: Python,
    SupportedPackageManagers.GO.value: Go,
    SupportedPackageManagers.NODEJS.value: NodeJS,
}


class Rubrical:
    configuration: RubricalConfig
    package_managers: List[BasePackageManager]
    repository_path: Path
    debug: bool

    def __init__(
        self, configuration: RubricalConfig, repository_path: Path, debug: bool = False
    ) -> None:
        self.configuration = configuration
        self.repository_path = repository_path
        self.debug = debug
        self.package_managers = []

        for package_manager in self.configuration.package_managers:
            self.package_managers.append(
                PACKAGE_MANAGER_MAPPING[package_manager.name]()  #  type:  ignore
            )

    def check_package_managers(
        self,
    ) -> Tuple[bool, bool, Dict[str, List[PackageCheckResult]]]:
        warnings_found = False
        blocks_found = False
        results: Dict[str, List[PackageCheckResult]] = {}

        for package_manager in self.package_managers:
            check_results = self.check_package_manager(package_manager)
            terminal.terminal_report(package_manager.name, check_results)

            if (
                any([x.check == PackageCheck.BLOCK for x in check_results])
                and not blocks_found
            ):
                blocks_found = True

            if (
                any([x.check == PackageCheck.WARN for x in check_results])
                and not warnings_found
            ):
                warnings_found = True

            results[package_manager.name] = check_results

        return (warnings_found, blocks_found, results)

    def check_package_manager(
        self, package_manager: BasePackageManager
    ) -> List[PackageCheckResult]:
        console.print_header(f"Grading {package_manager.name}", "🈴")
        check_results: List[PackageCheckResult] = []

        package_manager.read_package_manager_files(self.repository_path)
        package_manager.parse_package_manager_files()

        [configuration] = [
            x
            for x in self.configuration.package_managers
            if x.name == package_manager.name
        ]

        for file in package_manager.packages.keys():
            if self.debug:
                console.print_debug(f"Processing {file}.")

            for package in package_manager.packages[file]:
                for package_requirements in configuration.packages:
                    if self.debug:
                        console.print_debug(
                            f"Checking {package.name} @ {package.raw_constraint}."
                        )

                    if package_requirements.name == package.name:
                        check_results.append(
                            PackageCheckResult(
                                name=package.name,
                                file=file,
                                check=check_package(package_requirements, package),
                                version_package=package.raw_constraint,
                                version_warn=package_requirements.warn,
                                version_block=package_requirements.block,
                            )
                        )
                        if self.debug:
                            console.print_debug(
                                f"Check result for {package.name} @ {package.raw_constraint} is {check_results[-1].check.value}."
                            )

        return check_results
